<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/10/16
 * Time: 11:47 AM
 */


namespace qfq;

require_once(__DIR__ . '/../../qfq/Constants.php');

/**
 * Class OnArray
 * @package qfq
 */
class OnArray {

    /**
     * Builds a string from an assoc array.
     *
     * key/value are combined with $keyValueGlue.
     * values are enclosed by $encloseValue.
     * rows combined with $rowGlue
     *
     * @param array $dataArray
     * @param string $keyValueGlue
     * @param string $rowGlue
     * @return string
     */
    public static function toString(array $dataArray, $keyValueGlue = '=', $rowGlue = '&', $encloseValue = '') {
        $dataString = '';

        foreach ($dataArray as $key => $value) {
            $dataString .= $key . $keyValueGlue . $encloseValue . $value . $encloseValue . $rowGlue;
        }

        $glueLength = strlen($rowGlue);
        return substr($dataString, 0, strlen($dataString) - $glueLength);
    }

    /**
     * Sort array by array keys
     *
     * @param array $a
     */
    public static function sortKey(array &$a) {

        $result = array();

        $keys = array_keys($a);
        sort($keys);
        foreach ($keys as $key) {
            $result[$key] = $a[$key];
        }
        $a = $result;

        return;
    }

    /**
     * Trim all elemements in an array.
     * The array has to be a 1-dimensional array.
     *
     * @param array $arr
     * @param string $character_mask
     * @return array
     */
    public static function trimArray(array $arr, $character_mask) {
        foreach ($arr as $key => $item) {
            $arr[$key] = trim($item, $character_mask);
        }
        return $arr;
    }

    /**
     * Iterates over all records and return those with $row[$column]==$value
     *
     * @param array $dataArray
     * @param $column
     * @param $value
     * @return array
     */
    public static function filter(array $dataArray, $column, $value) {
        $result = array();

        foreach ($dataArray as $row) {
            if (isset($row[$column]) && $row[$column] == $value) {
                $result[] = $row;
            }
        }
        return $result;
    }

    /**
     * @return array
     */
    public static function inputCheckPatternArray() {
        return [
            'min|max' => 'min="%s"|max="%s"',
            'pattern' => 'pattern="%s"',
            'number' => 'pattern="^[0-9]*$"',
            'email' => 'pattern="^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})$"'
        ];
    }
}