<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/2/16
 * Time: 10:57 PM
 */

namespace qfq;

use qfq;
use qfq\CodeException;

require_once(__DIR__ . '/../../qfq/Constants.php');

/**
 * Class Sanatize
 * @package qfq
 */
class Sanatize {

    private function __construct() {
        // Class should never be instantiated
    }

    /**
     * Sanatize: check value by sanatize class. Return verified value or empty string if failed.
     *
     * @param string $value - value to check
     * @param string $sanatizeClass - class used to check
     * @return string - if check failed: empty string, if check is ok: value
     * @throws CodeException
     */
    public static function sanatize($value, $sanatizeClass = SANATIZE_DEFAULT) {

        switch ($sanatizeClass) {
            case SANATIZE_DEFAULT:
            case SANATIZE_ALLOW_DIGIT:
                if (!is_numeric($value)) {
                    $value = '';
                }
                break;

            case SANATIZE_ALLOW_ALNUMX:
                // replace ALNUMX valid characters, which are not part of ctype_alnum, by valid ctype_alnum characters, to fake ctype_alnum
                // definetely forbidden: ' " \ % $
                if (!ctype_alnum(str_replace(array('@', '-', '_', '.', ',', ';', ' ', '/', '(', ')'), 'a', $value)))
                    $value = '';
                break;

            case SANATIZE_ALLOW_ALL:
                break;

            default:
                throw new CodeException("Sanatize class '$sanatizeClass' unknown. Used to sanatize GET/POST Variable.", ERROR_UNKNOW_SANATIZE_CLASS);
                break;
        }

        return ($value);
    }

}