/**
 * @author Rafael Ostertag <rafael.ostertag@math.uzh.ch>
 */

/* global $ */
/* global console */

if (!QfqNS) {
    var QfqNS = {};
}

(function (n) {
    'use strict';

    n.QfqPage = function (settings) {
        this.settings = $.extend(
            {
                tabsId: "qfqTabs",
                formId: "qfqForm",
                submitTo: "typo3conf/ext/qfq/qfq/api/save.php",
                deleteUrl: "typo3conf/ext/qfq/qfq/api/delete.php",
                pageState: new QfqNS.PageState()
            }, settings
        );

        try {
            this.bsTabs = new QfqNS.BSTabs(this.settings.tabsId);

            var currentState = this.settings.pageState.getPageState();
            if (currentState !== "") {
                this.bsTabs.activateTab(currentState);
                QfqNS.PageTitle.setSubTitle(this.bsTabs.getTabName(currentState));
            } else {
                this.settings.pageState.setPageState(this.bsTabs.getCurrentTab(), QfqNS.PageTitle.get());
            }


            this.bsTabs.addTabShowHandler(this.tabShowHandler.bind(this));
            this.settings.pageState.addStateActivationHandler(this.popStateHandler.bind(this));
        } catch (e) {
            QfqNS.Log.message(e.message);
            this.bsTabs = null;
        }

        try {
            this.qfqForm = new QfqNS.QfqForm(this.settings.formId, this.settings.submitTo, this.settings.deleteUrl);
            this.qfqForm.setBsTabs(this.bsTabs);
        } catch (e) {
            QfqNS.Log.error(e.message);
            this.qfqForm = null;
        }
    };

    n.QfqPage.prototype.tabShowHandler = function (bsTabs) {
        // tabShowHandler will be called every time the tab will be shown, regardless of whether or not this happens
        // because of BSTabs.activateTab() or user interaction.
        //
        // Therefore, we have to make sure, that tabShowHandler() does not save the page state while we're restoring
        // a previous state, i.e. we're called because of the popStateHandler() below.
        if (this.settings.pageState.inPoppingHandler) {
            QfqNS.Log.debug("Prematurely terminating QfqPage.tabShowHandler(): called due to page state" +
                " restoration.");
            return;
        }
        var currentTabId = bsTabs.getCurrentTab();
        QfqNS.Log.debug('Saving state: ' + currentTabId);
        QfqNS.PageTitle.setSubTitle(bsTabs.getTabName(currentTabId));
        this.settings.pageState.setPageState(currentTabId, QfqNS.PageTitle.get());

    };

    n.QfqPage.prototype.popStateHandler = function (pageState) {
        this.bsTabs.activateTab(pageState.getPageState());
        QfqNS.PageTitle.set(pageState.getPageData());
    };

})(QfqNS);